@@ -34,7 +34,7 @@ def v1_1to1_2(
         if "cwlVersion" in proc:
             del proc["cwlVersion"]
 
-    return doc, "v1.2.0-dev1"
+    return doc, "v1.2"
 
 
 def v1_0to1_1(
@@ -143,12 +143,14 @@ def v1_0to1_1(
 def v1_1_0dev1to1_1(
     doc: CommentedMap, loader: Loader, baseuri: str
 ) -> Tuple[CommentedMap, str]:  # pylint: disable=unused-argument
+    """Public updater for v1.1.0-dev1 to v1.1."""
     return (doc, "v1.1")
 
 
 def v1_2_0dev1todev2(
     doc: CommentedMap, loader: Loader, baseuri: str
 ) -> Tuple[CommentedMap, str]:  # pylint: disable=unused-argument
+    """Public updater for v1.2.0-dev1 to v1.2.0-dev2."""
     return (doc, "v1.2.0-dev2")
 
 
@@ -178,8 +180,21 @@ def v1_2_0dev2todev3(
 def v1_2_0dev3todev4(
     doc: CommentedMap, loader: Loader, baseuri: str
 ) -> Tuple[CommentedMap, str]:  # pylint: disable=unused-argument
+    """Public updater for v1.2.0-dev3 to v1.2.0-dev4."""
     return (doc, "v1.2.0-dev4")
 
+def v1_2_0dev4todev5(
+    doc: CommentedMap, loader: Loader, baseuri: str
+) -> Tuple[CommentedMap, str]:  # pylint: disable=unused-argument
+    """Public updater for v1.2.0-dev4 to v1.2.0-dev5."""
+    return (doc, "v1.2.0-dev5")
+
+def v1_2_0dev5to1_2(
+    doc: CommentedMap, loader: Loader, baseuri: str
+) -> Tuple[CommentedMap, str]:  # pylint: disable=unused-argument
+    """Public updater for v1.2.0-dev5 to v1.2."""
+    return (doc, "v1.2")
+
 
 ORDERED_VERSIONS = [
     "v1.0",
@@ -189,11 +204,14 @@ ORDERED_VERSIONS = [
     "v1.2.0-dev2",
     "v1.2.0-dev3",
     "v1.2.0-dev4",
+    "v1.2.0-dev5",
+    "v1.2",
 ]
 
 UPDATES = {
     u"v1.0": v1_0to1_1,
     u"v1.1": v1_1to1_2,
+    u"v1.2": None
 }  # type: Dict[str, Optional[Callable[[CommentedMap, Loader, str], Tuple[CommentedMap, str]]]]
 
 DEVUPDATES = {
@@ -201,14 +219,15 @@ DEVUPDATES = {
     u"v1.2.0-dev1": v1_2_0dev1todev2,
     u"v1.2.0-dev2": v1_2_0dev2todev3,
     u"v1.2.0-dev3": v1_2_0dev3todev4,
-    u"v1.2.0-dev4": None,
+    u"v1.2.0-dev4": v1_2_0dev4todev5,
+    u"v1.2.0-dev5": v1_2_0dev5to1_2,
 }  # type: Dict[str, Optional[Callable[[CommentedMap, Loader, str], Tuple[CommentedMap, str]]]]
 
 
 ALLUPDATES = UPDATES.copy()
 ALLUPDATES.update(DEVUPDATES)
 
-INTERNAL_VERSION = u"v1.2.0-dev4"
+INTERNAL_VERSION = u"v1.2"
 
 ORIGINAL_CWLVERSION = "http://commonwl.org/cwltool#original_cwlVersion"
 
@@ -280,7 +299,7 @@ def update(
     metadata: CommentedMap,
     update_to: Optional[str] = None,
 ) -> CommentedMap:
-
+    """Update a CWL document to 'update_to' (if provided) or INTERNAL_VERSION."""
     if update_to is None:
         update_to = INTERNAL_VERSION
 
