@@ -4,29 +4,30 @@ import tempfile
 from functools import partial
 from io import StringIO
 from tempfile import NamedTemporaryFile
+from typing import Dict
 
-import pytest
+import py.path
+import pytest  # type: ignore
+from ruamel import yaml
 
 import cwltool.pack
 import cwltool.workflow
-from cwltool import load_tool
 from cwltool.context import LoadingContext
 from cwltool.load_tool import fetch_document, resolve_and_validate_document
 from cwltool.main import main, make_relative, print_pack
-from cwltool.pathmapper import adjustDirObjs, adjustFileObjs
 from cwltool.resolver import tool_resolver
-from ruamel import yaml
+from cwltool.utils import adjustDirObjs, adjustFileObjs
 
 from .util import get_data, needs_docker
 
 
-def test_pack():
+def test_pack() -> None:
     loadingContext, workflowobj, uri = fetch_document(get_data("tests/wf/revsort.cwl"))
 
     with open(get_data("tests/wf/expect_packed.cwl")) as packed_file:
-        expect_packed = yaml.safe_load(packed_file)
+        expect_packed = yaml.main.safe_load(packed_file)
 
-    packed = cwltool.pack.pack(loadingContext.loader, uri, loadingContext.metadata)
+    packed = cwltool.pack.pack(loadingContext, uri)
     adjustFileObjs(
         packed, partial(make_relative, os.path.abspath(get_data("tests/wf")))
     )
@@ -40,7 +41,7 @@ def test_pack():
     assert packed == expect_packed
 
 
-def test_pack_input_named_name():
+def test_pack_input_named_name() -> None:
     loadingContext, workflowobj, uri = fetch_document(
         get_data("tests/wf/trick_revsort.cwl")
     )
@@ -48,12 +49,14 @@ def test_pack_input_named_name():
     loadingContext, uri = resolve_and_validate_document(
         loadingContext, workflowobj, uri
     )
-    processobj = loadingContext.loader.resolve_ref(uri)[0]
+    loader = loadingContext.loader
+    assert loader
+    loader.resolve_ref(uri)[0]
 
     with open(get_data("tests/wf/expect_trick_packed.cwl")) as packed_file:
-        expect_packed = yaml.round_trip_load(packed_file)
+        expect_packed = yaml.main.round_trip_load(packed_file)
 
-    packed = cwltool.pack.pack(loadingContext.loader, uri, loadingContext.metadata)
+    packed = cwltool.pack.pack(loadingContext, uri)
     adjustFileObjs(
         packed, partial(make_relative, os.path.abspath(get_data("tests/wf")))
     )
@@ -67,7 +70,7 @@ def test_pack_input_named_name():
     assert packed == expect_packed
 
 
-def test_pack_single_tool():
+def test_pack_single_tool() -> None:
     loadingContext, workflowobj, uri = fetch_document(
         get_data("tests/wf/formattest.cwl")
     )
@@ -75,20 +78,20 @@ def test_pack_single_tool():
     loadingContext, uri = resolve_and_validate_document(
         loadingContext, workflowobj, uri
     )
-    processobj = loadingContext.loader.resolve_ref(uri)[0]
+    loader = loadingContext.loader
+    assert loader
+    loader.resolve_ref(uri)[0]
 
-    packed = cwltool.pack.pack(loadingContext.loader, uri, loadingContext.metadata)
+    packed = cwltool.pack.pack(loadingContext, uri)
     assert "$schemas" in packed
 
 
-def test_pack_fragment():
+def test_pack_fragment() -> None:
     with open(get_data("tests/wf/scatter2_subwf.cwl")) as packed_file:
-        expect_packed = yaml.safe_load(packed_file)
+        expect_packed = yaml.main.safe_load(packed_file)
 
     loadingContext, workflowobj, uri = fetch_document(get_data("tests/wf/scatter2.cwl"))
-    packed = cwltool.pack.pack(
-        loadingContext.loader, uri + "#scatterstep/mysub", loadingContext.metadata
-    )
+    packed = cwltool.pack.pack(loadingContext, uri + "#scatterstep/mysub")
     adjustFileObjs(
         packed, partial(make_relative, os.path.abspath(get_data("tests/wf")))
     )
@@ -99,8 +102,8 @@ def test_pack_fragment():
     )
 
 
-def test_pack_rewrites():
-    rewrites = {}
+def test_pack_rewrites() -> None:
+    rewrites = {}  # type: Dict[str, str]
 
     loadingContext, workflowobj, uri = fetch_document(
         get_data("tests/wf/default-wf5.cwl")
@@ -109,10 +112,12 @@ def test_pack_rewrites():
     loadingContext, uri = resolve_and_validate_document(
         loadingContext, workflowobj, uri
     )
-    processobj = loadingContext.loader.resolve_ref(uri)[0]
+    loader = loadingContext.loader
+    assert loader
+    loader.resolve_ref(uri)[0]
 
     cwltool.pack.pack(
-        loadingContext.loader, uri, loadingContext.metadata, rewrite_out=rewrites,
+        loadingContext, uri, rewrite_out=rewrites,
     )
 
     assert len(rewrites) == 6
@@ -124,8 +129,8 @@ cwl_missing_version_paths = [
 ]
 
 
-@pytest.mark.parametrize("cwl_path", cwl_missing_version_paths)
-def test_pack_missing_cwlVersion(cwl_path):
+@pytest.mark.parametrize("cwl_path", cwl_missing_version_paths)  # type: ignore
+def test_pack_missing_cwlVersion(cwl_path: str) -> None:
     """Ensure the generated pack output is not missing the `cwlVersion` in case of single tool workflow and single step workflow."""
     # Testing single tool workflow
     loadingContext, workflowobj, uri = fetch_document(get_data(cwl_path))
@@ -133,34 +138,38 @@ def test_pack_missing_cwlVersion(cwl_path):
     loadingContext, uri = resolve_and_validate_document(
         loadingContext, workflowobj, uri
     )
-    processobj = loadingContext.loader.resolve_ref(uri)[0]
+    loader = loadingContext.loader
+    assert loader
+    loader.resolve_ref(uri)[0]
 
     # generate pack output dict
-    packed = json.loads(print_pack(loadingContext.loader, uri, loadingContext.metadata))
+    packed = json.loads(print_pack(loadingContext, uri))
 
     assert packed["cwlVersion"] == "v1.0"
 
 
-def test_pack_idempotence_tool():
+def test_pack_idempotence_tool() -> None:
     """Ensure that pack produces exactly the same document for an already packed CommandLineTool."""
     _pack_idempotently("tests/wf/hello_single_tool.cwl")
 
 
-def test_pack_idempotence_workflow():
+def test_pack_idempotence_workflow() -> None:
     """Ensure that pack produces exactly the same document for an already packed workflow."""
     _pack_idempotently("tests/wf/count-lines1-wf.cwl")
 
 
-def _pack_idempotently(document):
+def _pack_idempotently(document: str) -> None:
     loadingContext, workflowobj, uri = fetch_document(get_data(document))
     loadingContext.do_update = False
     loadingContext, uri = resolve_and_validate_document(
         loadingContext, workflowobj, uri
     )
-    processobj = loadingContext.loader.resolve_ref(uri)[0]
+    loader = loadingContext.loader
+    assert loader
+    loader.resolve_ref(uri)[0]
 
     # generate pack output dict
-    packed_text = print_pack(loadingContext.loader, uri, loadingContext.metadata)
+    packed_text = print_pack(loadingContext, uri)
     packed = json.loads(packed_text)
 
     tmp = NamedTemporaryFile(mode="w", delete=False)
@@ -174,10 +183,12 @@ def _pack_idempotently(document):
         loadingContext, uri2 = resolve_and_validate_document(
             loadingContext, workflowobj, uri2
         )
-        processobj = loadingContext.loader.resolve_ref(uri2)[0]
+        loader2 = loadingContext.loader
+        assert loader2
+        loader2.resolve_ref(uri2)[0]
 
         # generate pack output dict
-        packed_text = print_pack(loadingContext.loader, uri2, loadingContext.metadata)
+        packed_text = print_pack(loadingContext, uri2)
         double_packed = json.loads(packed_text)
     finally:
         os.remove(tmp.name)
@@ -192,9 +203,11 @@ cwl_to_run = [
 ]
 
 
-@needs_docker
-@pytest.mark.parametrize("wf_path,job_path,namespaced", cwl_to_run)
-def test_packed_workflow_execution(wf_path, job_path, namespaced, tmpdir):
+@needs_docker  # type: ignore
+@pytest.mark.parametrize("wf_path,job_path,namespaced", cwl_to_run)  # type: ignore
+def test_packed_workflow_execution(
+    wf_path: str, job_path: str, namespaced: bool, tmpdir: py.path.local
+) -> None:
     loadingContext = LoadingContext()
     loadingContext.resolver = tool_resolver
     loadingContext, workflowobj, uri = fetch_document(get_data(wf_path), loadingContext)
@@ -202,8 +215,10 @@ def test_packed_workflow_execution(wf_path, job_path, namespaced, tmpdir):
     loadingContext, uri = resolve_and_validate_document(
         loadingContext, workflowobj, uri
     )
-    processobj = loadingContext.loader.resolve_ref(uri)[0]
-    packed = json.loads(print_pack(loadingContext.loader, uri, loadingContext.metadata))
+    loader = loadingContext.loader
+    assert loader
+    loader.resolve_ref(uri)[0]
+    packed = json.loads(print_pack(loadingContext, uri))
 
     assert not namespaced or "$namespaces" in packed
 
